extern "C" {
#include <openssl/bio.h>
#include <openssl/pkcs7.h>
#include <openssl/err.h>
#include <openssl/pem.h>
#include <openssl/cms.h>
}
//#ifdef WIN32
//#pragma warning(push)
//#pragma warning(disable: 4996) //4996 for _CRT_SECURE_NO_WARNINGS equivalent
//#include <openssl/applink.c>
//#pragma warning(pop)
//#endif
//**************************************************************************
//
//**************************************************************************
int main(void)
{
   // should not be needed for 1.1.0, according to docs; uncommenting does not change anything
   /*OPENSSL_init_crypto(
      OPENSSL_INIT_LOAD_CRYPTO_STRINGS |
      OPENSSL_INIT_ADD_ALL_CIPHERS |
      OPENSSL_INIT_ADD_ALL_DIGESTS |
      OPENSSL_INIT_NO_LOAD_CONFIG |
      OPENSSL_INIT_ASYNC, 
      NULL
   );*/
   // next uncomment next lines when using 1.0.2
   /*OpenSSL_add_all_algorithms();
   ERR_load_BIO_strings();
   ERR_load_crypto_strings();*/

   const EVP_MD *digest = EVP_get_digestbyname("SHA256");

   // load local certificate (used in the signing)
   BIO *localCertBio = BIO_new(BIO_s_file());
   BIO_read_filename(localCertBio, "local.cert");
   X509 *localCert = PEM_read_bio_X509(localCertBio, NULL, 0, NULL);
      
   // load private key for signing
   FILE *pkeyFile = NULL;
   EVP_PKEY *pKey = EVP_PKEY_new();
#ifdef WIN32
   fopen_s(&pkeyFile, "local.key", "r");
#else
   pkeyFile = fopen64("local.key", "r");
#endif
   PEM_read_PrivateKey(pkeyFile, &pKey, NULL, NULL);
   fclose(pkeyFile);

   // load local (for beeing able to decrypt outselfs) cert to stack
   stack_st_X509 *remoteCertStack = sk_X509_new_null();
   sk_X509_push(remoteCertStack, localCert);
   stack_st_X509 *localCertStack = sk_X509_new_null();
   sk_X509_push(localCertStack, localCert);

   // load data
   BIO *bioClearText = NULL;
   bioClearText = BIO_new_file("testfile.txt", "r");

   const int flags = PKCS7_BINARY;

   // sign data
   PKCS7 *signedData = PKCS7_sign(localCert, pKey, localCertStack, bioClearText, flags);
   
   // digest setzten wegen -md sha256
   PKCS7_set_digest(signedData, digest);

   // signed data to memory, we don't care 'bout that file
   BIO *bioSignedData = NULL;
   bioSignedData = BIO_new(BIO_s_mem());
   i2d_PKCS7_bio(bioSignedData, signedData);

   // Free cleartext...
   BIO_set_close(bioClearText, BIO_CLOSE);
   BIO_free(bioClearText);

   // encrypt the data
   PKCS7 *cryptData = NULL;
   cryptData = PKCS7_encrypt(remoteCertStack, bioSignedData, EVP_aes_256_cbc(), flags);

   // write to file in DER-Format
   BIO *bioCryptedData = NULL;
   bioCryptedData = BIO_new_file("testfile.crypt", "wb");
   i2d_PKCS7_bio(bioCryptedData, cryptData);

   // Free singed and encrypted
   BIO_set_close(bioSignedData, BIO_CLOSE);
   BIO_free(bioSignedData);

   BIO_set_close(bioCryptedData, BIO_CLOSE);
   BIO_free(bioCryptedData);

   return 0;
}
